#! /usr/bin/env node
/*
 * @Author: kael 
 * @Date: 2018-03-28 15:41:27 
 * @Last Modified by: kael
 * @Last Modified time: 2020-08-17 22:15:27
 */

require('./help');
const MA = require('./ma');
const http = require('http');
const iconv = require('iconv-lite');
const notifier = require('node-notifier');
const colors = require('colors');
const GREEN = colors.green;
const REEED = colors.red;

const stockrc = require('rc')('stock');
const follow = stockrc.f || stockrc.follow || '';
const movingAverage = stockrc.m || stockrc['moving-average'];
const follows = ('' + follow).trim().split(',').filter(id => id);

const notify = parseInt(stockrc.n || stockrc.notify, 10) || 60;
const timer = parseInt(stockrc.t || stockrc.timer, 10) || 1000;

const stocks_indexs = Object.keys({
  'sh000001': '上证指数',
  'sz399001': '深证成指',
});
stocks_indexs.forEach(stock => !follows.includes(stock) && follows.unshift(stock));

console.reset = function() {
  return process.stdout.write('\033c\n');
};

// [9:30, 11:30] + [13:00, 15:00]
function isOpen(date = new Date()) {
  let now = Date.now();

  let AM_start = date.setHours(9, 30, 0, 0);
  let AM_end = date.setHours(11, 30, 0, 0);
  let AM_open = now >= AM_start && now <= AM_end;

  let PM_start = date.setHours(13, 0, 0, 0);
  let PM_end = date.setHours(15, 0, 0, 0);
  let PM_open = now >= PM_start && now <= PM_end;
  return (AM_open || PM_open);
}

// http://qt.gtimg.cn/q=sz000001,sh000001
const query = async (codes) => {
  let options = {
    hostname: 'qt.gtimg.cn',
    path: `/q=${codes.join(',')}`,
  };

  const parse = (str) => {
    return ('' + str).replace(/[;\s]+$/, '').split(';').map(item => {
      let temp = {};
      let iraw = ('' + item).split('="');
      temp.stock_code = iraw[0].replace(/^[\s\n]*v_/, '');
      let values = iraw[1].split('~');
      [
        '市场', '名字', '代码', '当前价格', '昨收', '今开', '成交量', '外盘', '内盘',
        '买一', '买一量', '买二', '买二量', '买三', '买三量', '买四', '买四量', '买五', '买五量',
        '卖一', '卖一量', '卖二', '卖二量', '卖三', '卖三量', '卖四', '卖四量', '卖五', '卖五量',
        '最近逐笔成交', '时间', '涨跌', '涨跌%', '最高', '最低', '价格/成交量/成交额', '成交量', '成交额',
        '换手率', '市盈率', '未知', '最高', '最低', '振幅', '流通市值', '总市值', '市净率', '涨停价', '跌停价'
      ].forEach(name => {
        temp[name] = values.shift() || '';
      });
      temp['最近逐笔成交'] = ('' + temp['最近逐笔成交']).split('|');
      return temp;
    });
  };

  return new Promise((resolve, reject) => {
    http.request(options, function(resp) {
      let chunks = [];

      resp.on('error', function(error) {
        reject(error);
      });

      resp.on('data', function(chunk) {
        chunks.push(chunk);
      });

      resp.on('end', function() {
        let charset = ((resp.headers['content-type'] || '').match(/charset=(\S+)/) || [])[1] || 'GBK';
        let data = iconv.decode(Buffer.concat(chunks), charset);
        resolve(parse(data));
      });

    }).end();
  });
};

const fix = (name = '') => {
  let len = 0;
  ('' + name).split('').forEach(ch => {
    len += /[^\x00-\xff]/.test(ch) ? 2 : 1;
  });
  return name + '                '.substr(0, 16 - Math.max(len, 0));
};

let quote = [];
const start = (isboot) => {
  if (!isboot && !isOpen()) return;
  query(follows)
    .then(data => {
      quote = [];
      console.reset();
      console.log((new Date()).toTimeString().split(' ')[0]);
      console.log('=================================================================');
      console.log(`名称\t\t价格\t\t涨跌%\t\t成交额(万)`);
      data.forEach(item => {
        let color = item['涨跌'] > 0 ? REEED : GREEN;
        let apple = item['涨跌'] > 0 ? '🍎' : '🍏';
        quote.push(`${apple} ${item['名字']} ${item['涨跌%']}%`);
        console.log(color(`${fix(item['名字'])}${fix(item['当前价格'])}${fix(item['涨跌%'] + '%')}${fix(item['成交额'])}`));
      });
      console.log('=================================================================');
      let total = data
        .filter(v => stocks_indexs.includes(v.stock_code))
        .reduce((sum, v) => sum += parseInt(v['成交额'], 10), 0);
      console.log(colors.blue(`总成交额: ${total / 1e4} 亿`));
    });
  return start;
};

const notifys = () => {
  if (notify <= 0) return;
  return setInterval(() => {
    isOpen() && notifier.notify({
      title: `Stocklive ${(new Date()).toTimeString().split(' ')[0]}`,
      message: quote.join(' | '),
      timeout: 5,
    });
  }, notify * 1000);
}

const markets = [
  ['sh', 'sz'], // 沪深
  ['hk'],       // 港股
  ['us'],       // 美股
];
if (stockrc.stocks && movingAverage) {
  let mkts = movingAverage
    .toString(2).padStart(3, 0).slice(0, 3).split('')
    .reduce((acc, val, idx) => +val ? acc.concat(markets[idx]) : acc, []);
  stockrc.stocks = stockrc.stocks.filter(([_, id]) => mkts.includes(id.substring(0, 2)));
  MA(stockrc.stocks), setInterval(() => MA(stockrc.stocks), notify * 1000);
} else {
  setInterval(start(true), timer), notifys();
}

// Exception Handler
process.on('unhandledRejection', () => { });
process.on('uncaughtException', () => { });
